﻿;-- TIME_STAMP   2023-01-22 16:00:00   v 1.4

#include <APISysConstants.au3>
#include <WindowsConstants.au3>
#include <Array.au3>
#include <WinAPIInternals.au3>
#include <WinAPIShellEx.au3>
#include <WinAPISys.au3>

OnAutoItExitRegister('__MouseHover_OnAutoItExit')

Global Const $__g_sMouseHoverVersion = '1.4.0.0'
Global Const $__g_sMouseHoverDate = '2023/01/22 16:00:00'

Global $__g_bMouseTrack = True
Global $__g_hMouseHoverProc = DllCallbackRegister('__MouseHover_Proc', 'lresult', 'hwnd;uint;wparam;lparam;uint_ptr;dword_ptr')
Global $__g_pMouseHoverProc = DllCallbackGetPtr($__g_hMouseHoverProc)
Global $__g_aMouseHoverCtrl[0][5]

; #FUNCTION# ====================================================================================================================
; Name ..........: _MouseHover_Register
; Syntax ........: _MouseHover_Register($hWnd, $HoverFunc, $LeaveFunc[, $iTime])
; Description ...: Registriert ein Control-Element fuer den Hover-Effekt.
; Parameter .....: $hWnd = Control-Handle/ID ("-1" fuer zuletzt erstelltes Element)
;                  $HoverFunc = die Funktion, die beim "Hover" aufgerufen wird
;                  $LeaveFunc = die Funktion, die beim "Leave" aufgerufen wird
;                  $iTime = Zeit (in Millisekunden), bis der Hover-Effekt aktiv wird.
;                           Bei "-1" wird die Standard-Systemzeit verwendet
; Return Value(s): Bei Erfolg = True
;                  Bei Fehlern = False und @error <> 0
;                                @error = 1 $hWnd ist kein/e Control-Handle/ID
;                                @error = 2 Control ist bereits registriert
;                                @error = 3 $HoverFunc ist keine Funktion
;                                @error = 4 $LeaveFunc ist keine Funktion
;                                @error = 5 $iTime ist kein Integer
; Author(s) .....: Oscar (www.autoit.de)
;                  Bitnugger (www.autoit.de)
; ===============================================================================================================================
Func _MouseHover_Register($hWnd, $HoverFunc, $LeaveFunc, $iTime = -1)
	If Not IsHWnd($hWnd) Then $hWnd = GUICtrlGetHandle($hWnd)
	If Not IsHWnd($hWnd) Then Return SetError(1, 0, False)
	If _ArraySearch($__g_aMouseHoverCtrl, $hWnd) > -1 Then Return SetError(2, 0, False)
	If Not IsFunc($HoverFunc) Then Return SetError(3, 0, False)
	If Not IsFunc($LeaveFunc) Then Return SetError(4, 0, False)
	If Not IsInt($iTime) Then Return SetError(5, 0, False)
	Local $iCount = UBound($__g_aMouseHoverCtrl), $idSubClass = 100000
	If $iCount Then
		While _ArraySearch($__g_aMouseHoverCtrl, $idSubClass, 0, 0, 0, 0, 1, 4) > -1
			$idSubClass += 1
		WEnd
	EndIf
	ReDim $__g_aMouseHoverCtrl[$iCount + 1][UBound($__g_aMouseHoverCtrl, 2)]
	$__g_aMouseHoverCtrl[$iCount][0] = $hWnd
	$__g_aMouseHoverCtrl[$iCount][1] = $HoverFunc
	$__g_aMouseHoverCtrl[$iCount][2] = $LeaveFunc
	$__g_aMouseHoverCtrl[$iCount][3] = $iTime
	$__g_aMouseHoverCtrl[$iCount][4] = $idSubClass
	_WinAPI_SetWindowSubclass($hWnd, $__g_pMouseHoverProc, $idSubClass, 0)
	Return True
EndFunc   ;==>_MouseHover_Register

; #FUNCTION# ====================================================================================================================
; Name ..........: _MouseHover_Update
; Syntax ........: _MouseHover_Update($hWnd, $HoverFunc, $LeaveFunc[, $iTime])
; Description ...: Ermoeglicht das Aendern der Parameter fuer ein Control-Element,
;                  dass fuer den Hover-Effekt registriert wurde.
; Parameter .....: $hWnd = Control-Handle/ID ("-1" fuer zuletzt erstelltes Element)
;                  $HoverFunc = die Funktion, die beim "Hover" aufgerufen wird
;                  $LeaveFunc = die Funktion, die beim "Leave" aufgerufen wird
;                  $iTime = Zeit (in Millisekunden), bis der Hover-Effekt aktiv wird.
;                           Bei "-1" wird die Standard-Systemzeit verwendet
; Return Value(s): Bei Erfolg = True
;                  Bei Fehlern = False und @error <> 0
;                                @error = 1 $hWnd ist kein/e Control-Handle/ID
;                                @error = 2 Control ist nicht registriert
;                                @error = 3 $HoverFunc ist keine Funktion
;                                @error = 4 $LeaveFunc ist keine Funktion
;                                @error = 5 $iTime ist kein Integer
; Author(s)......: Oscar (www.autoit.de)
;                  Bitnugger (www.autoit.de)
; ===============================================================================================================================
Func _MouseHover_Update($hWnd, $HoverFunc, $LeaveFunc, $iTime = -1)
	If Not IsHWnd($hWnd) Then $hWnd = GUICtrlGetHandle($hWnd)
	If Not IsHWnd($hWnd) Then Return SetError(1, 0, False)
	Local $iIndex = _ArraySearch($__g_aMouseHoverCtrl, $hWnd)
	If @error Then Return SetError(2, 0, False)
	If Not IsFunc($HoverFunc) Then Return SetError(3, 0, False)
	If Not IsFunc($LeaveFunc) Then Return SetError(4, 0, False)
	$__g_aMouseHoverCtrl[$iIndex][1] = $HoverFunc
	$__g_aMouseHoverCtrl[$iIndex][2] = $LeaveFunc
	$__g_aMouseHoverCtrl[$iIndex][3] = $iTime
	Return True
EndFunc   ;==>_MouseHover_Update

; #FUNCTION# ====================================================================================================================
; Name ..........: _MouseHover_UnRegister
; Syntax ........: _MouseHover_UnRegister($hWnd)
; Description  ..: Entfernt den Hover-Effekt vom uebergebenen Control-Element
; Parameter .....: $hWnd = Control-Handle/ID ("-1" fuer zuletzt erstelltes Element)
; Return Value(s): Bei Erfolg = True
;                  Bei Fehlern = False und @error <> 0
;                                @error = 1 $hWnd ist kein/e Control-Handle/ID
;                                @error = 2 Control ist nicht registriert
; Author(s) .....: Oscar (www.autoit.de)
;                  Bitnugger (www.autoit.de)
; ===============================================================================================================================
Func _MouseHover_UnRegister($hWnd)
	If Not IsHWnd($hWnd) Then $hWnd = GUICtrlGetHandle($hWnd)
	If Not IsHWnd($hWnd) Then Return SetError(1, 0, False)
	Local $iIndex = _ArraySearch($__g_aMouseHoverCtrl, $hWnd)
	If $iIndex = -1 Then Return SetError(2, 0, False)
	_WinAPI_RemoveWindowSubclass($__g_aMouseHoverCtrl[$iIndex][0], $__g_pMouseHoverProc, $__g_aMouseHoverCtrl[$iIndex][4])
	_ArrayDelete($__g_aMouseHoverCtrl, $iIndex)
	$__g_bMouseTrack = True
	Return True
EndFunc   ;==>_MouseHover_UnRegister

; #INTERNAL_USE_ONLY#============================================================================================================
; Name ..........: __MouseHover_Proc
; Syntax ........: __MouseHover_Proc($hWnd, $iMsg, $wParam, $lParam, $iID, $pData)
; Description ...: Callback-Funktion fuer die Subclasses
;                  Hier wird auf die Mouse-Events reagiert und es werden die
;                  entsprechenden Funktionen aufgerufen.
; Author(s) .....: Oscar (www.autoit.de)
;                  Bitnugger (www.autoit.de)
; ===============================================================================================================================
Func __MouseHover_Proc($hWnd, $iMsg, $wParam, $lParam, $iID, $pData)
	#forceref $iID, $pData
	Local $iIndex = _ArraySearch($__g_aMouseHoverCtrl, $hWnd)
	If $iIndex > -1 Then
		Switch $iMsg
			Case $WM_MOUSEMOVE
				If $__g_bMouseTrack Then
					_WinAPI_TrackMouseEvent($hWnd, BitOR($TME_HOVER, $TME_LEAVE), $__g_aMouseHoverCtrl[$iIndex][3])
					$__g_bMouseTrack = False
				EndIf
			Case $WM_MOUSEHOVER
				$__g_aMouseHoverCtrl[$iIndex][1](_WinAPI_GetDlgCtrlID($hWnd))
			Case $WM_MOUSELEAVE
				$__g_bMouseTrack = True
				$__g_aMouseHoverCtrl[$iIndex][2](_WinAPI_GetDlgCtrlID($hWnd))
		EndSwitch
	EndIf
	Return _WinAPI_DefSubclassProc($hWnd, $iMsg, $wParam, $lParam)
EndFunc   ;==>__MouseHover_Proc

; #INTERNAL_USE_ONLY#============================================================================================================
; Name ..........: __MouseHover_OnAutoItExit
; Syntax ........: __MouseHover_OnAutoItExit()
; Description ...: Zum aufraeumen bei Programmende (Entfernen der Subclasses)
; Author(s) .....: Oscar (www.autoit.de)
;                  Bitnugger (www.autoit.de)
; ===============================================================================================================================
Func __MouseHover_OnAutoItExit()
	If UBound($__g_aMouseHoverCtrl) Then
		For $i = 0 To UBound($__g_aMouseHoverCtrl) - 1
			_WinAPI_RemoveWindowSubclass($__g_aMouseHoverCtrl[$i][0], $__g_pMouseHoverProc, $__g_aMouseHoverCtrl[$i][4])
		Next
	EndIf
	DllCallbackFree($__g_hMouseHoverProc)
EndFunc   ;==>__MouseHover_OnAutoItExit
